unit UpdateMulti;

INTERFACE

uses
  Windows, SysUtils, Classes, DB, DBTables;

type
  TUpdateMulti = class(TDataSetUpdateObject)
  private
    FDataSet: 			{$IFDEF VER90}TDataSet{$ELSE}TBDEDataSet{$ENDIF};
    FExecQuery:			TQuery;
    FOnDelete:			TNotifyEvent;
    FOnInsert:			TNotifyEvent;
    FOnModify:			TNotifyEvent;
    FParamQueries:	array [TUpdateKind] of TQuery;
    FSeparator:			String;
  protected
    function GetDataSet: {$IFDEF VER90}TDataSet{$ELSE}TBDEDataSet{$ENDIF}; override;
    procedure SetDataSet(ADataSet: {$IFDEF VER90}TDataSet{$ELSE}TBDEDataSet{$ENDIF}); override;
		function GetParamsProp(Index: Integer) : TParams;
    function GetQuery(UpdateKind: TUpdateKind): TQuery;
    function GetSQL(Index: Integer): TStrings;
		procedure SetParamsProp(Index: Integer; Value: TParams);
    procedure SetSeparator(Value: String);
    procedure SetSQL(Index: Integer; Value: TStrings);

    procedure ExecSQL;
    procedure SetParams;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Apply(UpdateKind: TUpdateKind); override;

    property DataSet;
    property Queries[UpdateKind: TUpdateKind]: TQuery read GetQuery;
  published
    property DeleteParams: TParams index Ord(ukDelete) read GetParamsProp write SetParamsProp;
    property InsertParams: TParams index Ord(ukInsert) read GetParamsProp write SetParamsProp;
    property ModifyParams: TParams index Ord(ukModify) read GetParamsProp write SetParamsProp;

    property DeleteSQL: TStrings index Ord(ukDelete) read GetSQL write SetSQL;
    property InsertSQL: TStrings index Ord(ukInsert) read GetSQL write SetSQL;
    property ModifySQL: TStrings index Ord(ukModify) read GetSQL write SetSQL;

    property OnDelete: TNotifyEvent read FOnDelete write FOnDelete;
    property OnInsert: TNotifyEvent read FOnInsert write FOnInsert;
    property OnModify: TNotifyEvent read FOnModify write FOnModify;

    property Separator: String read FSeparator write SetSeparator;
  end;


IMPLEMENTATION


uses
	{$IFDEF VER90}DBConsts{$ELSE}BDEConst{$ENDIF}, Dialogs;


procedure CopyParams(Source, Dest: TParams);
var
	i: Integer;
begin
	for i := 0 to Dest.Count-1 do
	begin
    try
			if Source.ParamByName(Dest[i].Name).IsNull then
  	  	Continue;
			Dest[i].Assign(Source.ParamByName(Dest[i].Name));
    except
      // Pas d'arret sur exception qui signifie simplement que le
      // paramtre n'a pas t trouv.
    end;
	end;
end;


// TUpdateMulti

constructor TUpdateMulti.Create(AOwner: TComponent);
var
  uk: TUpdateKind;
begin
  inherited;
  FExecQuery := TQuery.Create(Self);
  for uk := Low(FParamQueries) to High(FParamQueries) do
    FParamQueries[uk] := TQuery.Create(Self);
  FSeparator := '!';
end;


destructor TUpdateMulti.Destroy;
begin
  if Assigned(FDataSet) and (FDataSet.UpdateObject = Self) then
    FDataSet.UpdateObject := nil;
  inherited;
end;


function TUpdateMulti.GetDataSet: {$IFDEF VER90}TDataSet{$ELSE}TBDEDataSet{$ENDIF};
begin
  Result := FDataSet;
end;


function TUpdateMulti.GetParamsProp(Index: Integer) : TParams;
begin
	Result := FParamQueries[TUpdateKind(Index)].Params;
end;


function TUpdateMulti.GetQuery(UpdateKind: TUpdateKind): TQuery;
begin
	Result := FParamQueries[UpdateKind];
end;


function TUpdateMulti.GetSQL(Index: Integer): TStrings;
begin
  Result := FParamQueries[TUpdateKind(Index)].SQL;
end;


procedure TUpdateMulti.SetDataSet(ADataSet: {$IFDEF VER90}TDataSet{$ELSE}TBDEDataSet{$ENDIF});
begin
  FDataSet := ADataSet;
  if (FDataSet is TDBDataSet) then
  begin
    FExecQuery.SessionName := TDBDataSet(FDataSet).SessionName;
    FExecQuery.DatabaseName := TDBDataSet(FDataSet).DataBaseName;
  end;
end;


procedure TUpdateMulti.SetParamsProp(Index: Integer; Value: TParams);
begin
	FParamQueries[TUpdateKind(Index)].Params.Assign(Value);
end;


procedure TUpdateMulti.SetSeparator(Value: String);
begin
	if Value = '' then
  	raise Exception.Create('The separator must be not null');
  FSeparator := Value;
end;


procedure TUpdateMulti.SetSQL(Index: Integer; Value: TStrings);
var
  i: Integer;
  Field: TField;
  Param: TParam;
  Params: TParams;
	ParamName: String;
  SelectQuery: TQuery;
  UpdateQuery: TQuery;
begin
  UpdateQuery := Queries[TUpdateKind(Index)];
  UpdateQuery.SQL.Assign(Value);
  SelectQuery := nil;
  if Assigned(Dataset) and (Dataset is TQuery) then
    SelectQuery := TQuery(Dataset);
  if SelectQuery = nil then
  	Exit;

  Params := UpdateQuery.Params;
  for i := 0 to Params.Count-1 do
  begin
    Param := Params[i];
    ParamName := Param.Name;
    if CompareText(Copy(ParamName, 1, 4), 'OLD_') = 0 then
    System.Delete(ParamName, 1, 4);
    Field := SelectQuery.FindField(ParamName);
    if Field <> nil then
      Param.DataType := Field.DataType;
  end;
end;


procedure TUpdateMulti.ExecSQL;
begin
  with FExecQuery do
  begin
		Prepare;
		ExecSQL;
		if RowsAffected <> 1 then
			{$IFDEF VER90}DBError(SUpdateFailed){$ELSE}DatabaseError(SUpdateFailed){$ENDIF};
	end;
end;


procedure TUpdateMulti.SetParams;
var
  I: Integer;
  Old: Boolean;
  Param: TParam;
  PName: string;
  Field: TField;
begin
  if not Assigned(FDataSet) then Exit;
  with FExecQuery do
  begin
    for I := 0 to Params.Count - 1 do
    begin
      Param := Params[I];
      PName := Param.Name;
      Old := CompareText(Copy(PName, 1, 4), 'OLD_') = 0;
      if Old then System.Delete(PName, 1, 4);
      Field := FDataSet.FindField(PName);
      if not Assigned(Field) then Continue;
      if Old then
        Param.AssignFieldValue(Field, Field.OldValue)
      else
        Param.AssignFieldValue(Field, Field.NewValue);
    end;
  end;
end;


procedure TUpdateMulti.Apply(UpdateKind: TUpdateKind);
var
  i: Integer;
	Stmt: String;
	StmtEnd: Integer;
  StmtLength: Integer;
begin
  for i := 0 to FParamQueries[UpdateKind].Params.Count-1 do
		FParamQueries[UpdateKind].Params[i].Clear;
	case UpdateKind of
  	ukDelete:	if Assigned(FOnDelete) then FOnDelete(Self);
    ukInsert:	if Assigned(FOnInsert) then FOnInsert(Self);
    ukModify:	if Assigned(FOnModify) then FOnModify(Self);
  end;

  Stmt := FParamQueries[UpdateKind].SQL.Text;

  repeat
  	StmtLength := Length(Stmt);
    StmtEnd := Pos(FSeparator+#13#10, Stmt);
	  if StmtEnd = 0 then
	  	StmtEnd := StmtLength;
		FExecQuery.Params.Clear;
    FExecQuery.SQL.Text := Copy(Stmt, 1, StmtEnd-1);
 	  Delete(Stmt, 1, StmtEnd+2);

    SetParams;
    CopyParams(FParamQueries[UpdateKind].Params, FExecQuery.Params);
 	  ExecSQL;
  until StmtEnd = StmtLength;
end;

end.

